@@ -9,21 +9,21 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.26.2-Buenos Aires"
-#define _QGIS_VERSION "3.26.2-Buenos Aires"
+#define VERSION "3.28.1-Firenze"
+#define _QGIS_VERSION "3.28.1-Firenze"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3262
+//#define VERSION_INT 3281
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 32602
-#define _QGIS_VERSION_INT 32602
-#define ABISYM(x) x ## 32602
+#define VERSION_INT 32801
+#define _QGIS_VERSION_INT 32801
+#define ABISYM(x) x ## 32801
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Buenos Aires"
+#define RELEASE_NAME "Firenze"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -31,7 +31,7 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.26.2"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.28.1"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
@@ -46,14 +46,14 @@
 
 #define QT_PLUGINS_DIR "/usr/lib/qt/plugins"
 
-#define PYTHON_VERSION "3.10.6"
+#define PYTHON_VERSION "3.10.8"
 #define PYTHON_VERSION_MAJOR "3"
 #define PYTHON_VERSION_MINOR "10"
-#define PYTHON_VERSION_PATCH "6"
+#define PYTHON_VERSION_PATCH "8"
 
 #define PROJ_VERSION_MAJOR 9
-#define PROJ_VERSION_MINOR 0
-#define PROJ_VERSION_PATCH 1
+#define PROJ_VERSION_MINOR 1
+#define PROJ_VERSION_PATCH 0
 
 /* #undef USING_NMAKE */
 
